  Function MinF!(a!,b!)
    If a<b Return a
    Return b
  EndFunction

  Function MaxF!(a!,b!)
    If a>b Return a
    Return b
  EndFunction

  Const D_Up = 0
  Const D_Left= 1
  Const D_Down = 2
  Const D_Right= 3

Type Actor
  Field x,y         'Position of the Actor
  Field Visual      'Visual of Actor
  Field Direction
  Field Frame       'Current Frame
  Field FrameCount  'Max frames this actor has default = 1
  Field FrameTimer
  Field FrameCurrent

  Method Update() Abstract
  Method Draw() Abstract
EndType

Type Player Extends Actor
  Field dy#=0,Jumping#=0,Idle=False

  Method Draw()
    Local xx,yy
    xx=x-MapOffX
    yy=y-MapOffY
    'Shadow first
    SetBlend ALPHABLEND
    SetColor 0,0,0
    SetAlpha 0.4
    DrawImage Visual,xx+8,yy-8,Frame
    SetBlend MASKBLEND
    SetColor 255,255,255
    SetAlpha 1
    DrawImage Visual,xx,yy,Frame

    'drawrect xx-2,yy,1,1
    'drawrect xx-2,yy+31,1,1
    'drawrect xx-2,yy+63,1,1
    'drawrect xx+34,yy,1,1
    'drawrect xx+34,yy+31,1,1
    'drawrect xx+34,yy+63,1,1
  EndMethod

  Method Fall()
    If CheckCollisionTile(x+8,y+65)=0 And CheckCollisionTile(x+23,y+63)=0
      If dy=0 dy=.01
        y:+dy
        dy:+dy
        If dy>8 dy=8
      EndIf

  EndMethod

  Method Update()
    Idle = True
    Local xx#,yy#
    Local wx=GraphicsWidth(),wy=GraphicsHeight()
    Fall()
    UpdateKeys()
    UpdateFrames()
    xx=x+16
    yy=y+32
    MapOffX = MaxF(XX-(wx/2),0)
    MapOffX = MinF(MapOffX,6400-wx)
    MapOffY = MaxF(YY-(wy/2),0)
    MapOffY = MinF(MapOffY,6400-wy)
  EndMethod

  Method UpdateKeys()
    If KeyHit(Key_Space) And (Jumping = 0 And dy =0)
      Jumping = 8
    EndIf
    If KeyDown(Key_Left)
      Idle=False
      If Not Direction = D_Left
        FrameCurrent=FrameTimer
        Direction = D_Left
      EndIf
      If CheckCollisionTile(x+6,y) Return
      If CheckCollisionTile(x+6,y+31) Return
      If CheckCollisionTile(x+6,y+62) Return
      x:-2
      If x<0 x=0
    ElseIf KeyDown(Key_Right)
      Idle=False
      If Not Direction = D_Right
        FrameCurrent=FrameTimer
        Direction = D_Right
      EndIf
      If CheckCollisionTile(x+24,y) Return
      If CheckCollisionTile(x+24,y+31) Return
      If CheckCollisionTile(x+24,y+62) Return
      x:+2
    EndIf

  EndMethod

  Method UpdateFrames()
    If Idle=False
      FrameCurrent:+1
      If FrameCurrent>=FrameTimer
        FrameCurrent=0
        Select Direction
          Case D_Left
            If frame = 2
              frame = 0
            Else
              frame = 2
            EndIf
          Case D_Right
            If frame = 3
              frame = 1
            Else
              frame = 3
            EndIf
        EndSelect
      EndIf
    EndIf
  EndMethod

  Function Create:Player( vis:TImage, x=320, y=240)
    Local p:Player = New Player
    p.Visual = vis
    p.FrameCount = FrameCount
    p.Frame = 0
    p.Direction = D_Right
    p.x = x
    p.y = y
    p.FrameTimer=10
    p.FrameCurrent=0

    Return p
  EndFunction
EndType


Global thePlayer:Player
Global imgList:TImage
Global plyImage:TImage
Global Map:Int[400,400,3]
Global MapOffX#=0,MapOffY#=0
Global ActorList:TList = New TList

Setup()


While Not KeyDown(Key_Escape)
  UpdateAction( ActorList )
  DrawAll
  Flip
  FlushMem
Wend

End

Function Setup()
  Graphics 1024,768
  LoadMap "media\maps\map1.test"
  ImgList:TImage = LoadAnimImage ("media\tiles\block1.png",32,32,0,380)
  PlyImage:TImage= LoadAnimImage ("media\Actors\dog.bmp",32,64,0,4)
  thePlayer = Player.Create(PlyImage,320,128)

  'Add the player to the actor list
  ActorList.AddLast(thePlayer)

End Function

Function DrawAll()
  Cls

  DrawMap( MapOffX, MapOffY ,0)
  DrawActors ( ActorList )
  DrawMap( MapOffX, MapOffY ,1)
EndFunction

Function UpdateAction( l:TList )
  Local ac:Actor
  For ac=EachIn l
    ac.Update()
  Next
EndFunction

Function DrawActors( l:TList )
  Local ac:Actor
  For ac=EachIn l
    ac.Draw()
  Next
EndFunction

'
' Draw a Map Layer
'
Function DrawMap(x_offset#,y_offset#, layer=0)
  SetBlend SOLIDBLEND
  Local s_Height = GraphicsHeight()
  Local s_Width = GraphicsWidth()
  Local tile:Int

  'screen position
  Local sx#,sy#

  'map tile pos
  Local tx#,ty#

  'preposition start x
  Local tilestartx# = Floor(x_offset / 32)
  Local startposx# = -(x_offset Mod 32)
	ty=Floor(y_offset/32)
	sy=-(y_offset Mod 32)

	While sy<s_height
    tx=tilestartx
		ty=ty Mod 200
		sx=startposx

		While sx<s_width
			If map[tx Mod 200,ty, 2] = layer
        tile=map[tx Mod 200,ty, 0]

  			If tile
  			DrawImage ImgList,sx,sy,tile
        EndIf
      EndIf
			tx=tx+1
      sx=sx+32
		Wend

		ty=ty+1
    sy=sy+32
	Wend

EndFunction


'
' Load in a map here
'
Function LoadMap(file$)
  str = ReadFile("littleendian::"+file$)
  For Y=0 To 199
    For x=0 To 199
      For b=0 To 2
        map[x,y,b] = Readint(str)
      Next
    Next
  Next
EndFunction

Function GetTile(x,y,layer=0)
  x:Shr 5
  y:Shr 5
  Return map[x Mod 200,y Mod 200,layer]
EndFunction

Function CheckCollisionTile(x,y)
  Local tile = GetTile(x,y,1)
  Return tile<>0
EndFunction
