Import "Vector3.bmx"

  Const MATRIX_AFTER = 1
  Const MATRIX_BEFORE= 0

  Const ShearX_Y = 0
  Const ShearX_Z = 1
  Const ShearY_X = 2
  Const ShearY_Z = 3
  Const ShearZ_X = 4
  Const ShearZ_Y = 5

  Type Matrix4
    field m#[16]
    
    Method this:Float Ptr()
      return m
    EndMethod

    Method Zero()
      local a
      for a=0 to 15
        m[a]=0
      next
    EndMethod
    Method Constant( v# )
      local a:int
      for a=0 to 15
        m[a]=v
      next
    EndMethod
    Method Copy(mt:Matrix4)
      local a:int
      for a=0 to 15
        m[a]=mt.m[a]
      next
    EndMethod
    Method Clone( p:Matrix4 var )
      local a:int
      for a=0 to 15
        p.m[a]=m[a]
      next
    EndMethod
    Method CopyM( f:float ptr )
      local a:int
      for a=0 to 15
        m[a]=f[a]
      next
    EndMethod

    Method Identity()
      Zero()
      m[0] = 1
      m[5] = 1
      m[10]= 1
      m[15]= 1
    EndMethod

    Method Transpose()
      local tmp#[16]
      local i:Int, j:Int
      For i=0 to 3
        For j=0 to 3
          tmp[4*i+j] = m[i+4*j]
        Next
      Next
      For i=0 to 15
        m[i] = tmp[i];
      next
    EndMethod
    Method TransposeTo(ans:Matrix4 var)
      local tmp#[16]
      local i:Int, j:Int
      For i=0 to 3
        For j=0 to 3
          tmp[4*i+j] = m[i+4*j]
        Next
      Next
      For i=0 to 15
        ans.m[i] = tmp[i];
      next
    EndMethod

    Method RotateX( angle# )
      local tmp:Matrix4 = New Matrix4
      tmp.RotationX(angle)
      Multiply(tmp)
    EndMethod
    Method RotateY( angle# )
      local tmp:Matrix4 = New Matrix4
      tmp.RotationY(angle)
      Multiply(tmp)
    EndMethod
    Method RotateZ( angle# )
      local tmp:Matrix4 = New Matrix4
      tmp.RotationZ(angle)
      Multiply(tmp)
    EndMethod

    Method Rotation( pitch#, yaw#, roll# )
      Identity()
      Local pmat:Matrix4 = New Matrix4
      Local ymat:Matrix4 = New Matrix4
      Local rmat:Matrix4 = New Matrix4
      pmat.RotationX( pitch )
      ymat.RotationY( yaw )
      rmat.RotationZ( roll )
      Multiply( ymat )
      Multiply( pmat )
      Multiply( rmat )
    EndMethod
    Method RotationX( angle# )
      Identity()
      m[5] = cos( angle )
      m[10]= m[5]
      m[6] = sin( angle )
      m[9] =-m[6]
    EndMethod
    Method RotationY( angle# )
      Identity()
      m[0] = cos( angle )
      m[2] =-sin( angle )
      m[8] =-m[2]
      m[10]= m[0]
    EndMethod
    Method RotationZ( angle# )
      Identity()
      m[0] = cos(angle)
      m[1] = sin(angle)
      m[4] =-m[1]
      m[5] = m[0]
    EndMethod
    Method Translation( x:Float, y:Float, z:Float )
      Identity()
      m[12] = x
      m[13] = y
      m[14] = z
    EndMethod

    Method Translate( x:Float, y:Float, z:Float )
      local tmp:Matrix4 = New Matrix4
      tmp.Translation( x,y,z )
      Multiply(tmp)
    EndMethod
    Method Scale( x:Float, y:Float, z:Float )
      Identity()
      m[0] = x
      m[5] = y
      m[10]= z
    EndMethod
    Method Shear( dir, amount# )
      Identity()
      select dir
        case ShearX_Y
          m[4] = amount
        case ShearX_Z
          m[8] = amount
        case ShearY_X
          m[1] = amount
        case ShearY_Z
          m[9] = amount
        case ShearZ_X
          m[2] = amount
        case ShearZ_Y
          m[6] = amount
      EndSelect
    EndMethod

    Method MulScalar( value# )
      local a:int
      for a=0 to 15
        m[a]=m[a]*value
      next
    EndMethod
    Method Multiply(m1:Matrix4)
      local i:int
      local j:int
      local tmat:Matrix4 = new Matrix4
      local tmp#[4]
      for j=0 to 3
        tmp[0] = m[j]
        tmp[1] = m[4+j]
        tmp[2] = m[8+j]
        tmp[3] = m[12+j]
        for i=0 to 3
          tmat.m[4*i+j] = m1.m[4*i]*tmp[0] + m1.m[4*i+1]*tmp[1] + m1.m[4*i+2]*tmp[2] + m1.m[4*i+3]*tmp[3]
        next
      next
      Copy(tmat)
    EndMethod

    Method MultiplyMatrix(ans:Matrix4, m1:Matrix4, when=MATRIX_AFTER )
      select when
        case MATRIX_BEFORE
          MultiplyMatrixBefore( ans , m1 )
        default
          MultiplyMatrixAfter( ans , m1 )
      endselect
    EndMethod
    Method MultiplyMatrixAfter(ans:Matrix4 var ,m1:Matrix4)
      local i:int
      local j:int
      local tmat:Matrix4 = new Matrix4
      local tmp#[4]
      for j=0 to 3
        tmp[0] = m[j]
        tmp[1] = m[4+j]
        tmp[2] = m[8+j]
        tmp[3] = m[12+j]
        for i=0 to 3
          tmat.m[4*i+j] = m1.m[4*i]*tmp[0] + m1.m[4*i+1]*tmp[1] + m1.m[4*i+2]*tmp[2] + m1.m[4*i+3]*tmp[3]
        next
      next
      ans.Copy(tmat)
    EndMethod
    Method MultiplyMatrixBefore(ans:Matrix4 var , m1:Matrix4 )
      local tmp:Matrix4 = new Matrix4
      local tmp2:Matrix4 = new Matrix4
      local a:Int
      for a=0 to 15
        tmp.m[a] = m[a]
      next
      tmp2.Copy(m1)
      tmp2.MultiplyMatrixAfter(tmp2,tmp)
      ans.copy(tmp2)
    EndMethod
    'Sets this matrix to represent a window perspective
     Method Window(lft#,rgt#,btm#,top#,near#,far#)
       Constant(0.0)
       m[0] = (2.0*near) / (rgt-lft)
       m[5] = (2.0*near) / (top-btm)
       m[8] = (rgt+lft) / (rgt-lft)
       m[9] = (top+btm) / (top-btm)
       m[10]= -(far+near) / (far-near)
       m[11]= -1.0
       m[14]= -(2.0*far*near) / (far-near)
     EndMethod
     'Sets this matrix to a 3D orthographic matrix
     Method Ortho(lft#, rgt#, btm#, top#, near#, far#)
       Constant(0.0) 'Initialize this matrix to 0
       m[0] =  2.0 / (rgt-lft)
       m[5] =  2.0 / (top-btm)
       m[10] = -2.0 / (far-near)
       m[12] = -(rgt+lft) / (rgt-lft)
       m[13] = -(top+bottom) / (top-btm)
       m[14] = -(far+near) / (far-near)
       m[15] = 1.0
     EndMethod
     'Sets this matrix to a 2D orthographic matrix
     Method Ortho2(lft#, rgt#, btm#, top#)
       Constant(0.0) 'initialize this matrix to 0
       m[0] = 2.0 / (rgt-lft)
       m[5] = 2.0 / (top-btm)
       m[10] =-1.0
       m[12] =-(rgt+lft) / (rgt-lft)
       m[13] =-(top+btm) / (top-btm)
       m[15] = 1.0
     EndMethod
    'Multiplies a 3D point n:TVector3 by the Matrix, returns in first ans
    Method MultiplyVector(ans:Vector3 var, n:Vector3)
      local tmp:Float[3]
      local tmp2:Float[3]
      local itmp3:Float
      itmp3 = 1.0 /(n.x*m[3]+n.y*m[7]+n.z*m[11]+m[15])
      tmp[0] = itmp3*n.x
      tmp[1] = itmp3*n.y
      tmp[2] = itmp3*n.z
      ans.x=tmp[0]*m[0]+tmp[1]*m[4]+tmp[2]*m[ 8]+itmp3*m[12]
      ans.y=tmp[0]*m[1]+tmp[1]*m[5]+tmp[2]*m[ 9]+itmp3*m[13]
      ans.z=tmp[0]*m[2]+tmp[1]*m[6]+tmp[2]*m[10]+itmp3*m[14]
    EndMethod
    'Multiplies a 3D norm (second arg) by the Matrix, returns in first arg
    'This differs from point multiplication in that the translatation operations
    'are ignored.
    Method MultiplyNormal(ans:Vector3 var, n:Vector3)
      local tmp:Float[3]
      local itmp:Float
      tmp[0]=n.x*m[0]+n.y*m[4] +n.z*m[8]
      tmp[1]=n.x*m[1]+n.y*m[5] +n.z*m[9]
      tmp[2]=n.x*m[2]+n.y*m[6] +n.z*m[10]
      itmp=1.0/sqr(tmp[0]*tmp[0]+tmp[1]*tmp[1]+tmp[2]*tmp[2])
      ans.x=tmp[0]*itmp
      ans.y=tmp[1]*itmp
      ans.z=tmp[2]*itmp
    EndMethod

    Method Invert(ans:Matrix4 var)
      local out:Matrix4 = new Matrix4
      local i,j
      local det!
      'Calculate the adjoint matrix */
      Adjoint( out )
      'Calculate the 4x4 determinant
      'if the determinant is zero,
      'then the inverse matrix is not unique.
      det = det4x4()
      if abs(det) < 0.001
        return true
      endif
      'Scale the adjoint matrix to get the inverse
      ans.Copy(out)
    EndMethod
    Method Adjoint( out:Matrix4 var )
      'row column labeling reversed since we transpose rows & columns */
      out.m[0]  =   det3x3( m[5], m[9], m[13], m[6], m[10], m[14], m[7], m[11], m[15])
      out.m[4]  = - det3x3( m[4], m[8], m[12], m[6], m[10], m[14], m[7], m[11], m[15])
      out.m[8]  =   det3x3( m[4], m[8], m[12], m[5], m[9], m[13], m[7], m[11], m[15])
      out.m[12] = - det3x3( m[4], m[8], m[12], m[5], m[9], m[13], m[6], m[10], m[14])

      out.m[1]  = - det3x3( m[1], m[9], m[13], m[2], m[10], m[14], m[3], m[11], m[15])
      out.m[5]  =   det3x3( m[0], m[8], m[12], m[2], m[10], m[14], m[3], m[11], m[15])
      out.m[9]  = - det3x3( m[0], m[8], m[12], m[1], m[9], m[13], m[3], m[11], m[15])
      out.m[13] =   det3x3( m[0], m[8], m[12], m[1], m[9], m[13], m[2], m[10], m[14])

      out.m[2]  =   det3x3( m[1], m[5], m[13], m[2], m[6], m[14], m[3], m[7], m[15])
      out.m[6]  = - det3x3( m[0], m[4], m[12], m[2], m[6], m[14], m[3], m[7], m[15])
      out.m[10] =   det3x3( m[0], m[4], m[12], m[1], m[5], m[13], m[3], m[7], m[15])
      out.m[14] = - det3x3( m[0], m[4], m[12], m[1], m[5], m[13], m[2], m[6], m[14])

      out.m[3]  = - det3x3( m[1], m[5], m[9], m[2], m[6], m[10], m[3], m[7], m[11])
      out.m[7]  =   det3x3( m[0], m[4], m[8], m[2], m[6], m[10], m[3], m[7], m[11])
      out.m[11] = - det3x3( m[0], m[4], m[8], m[1], m[5], m[9], m[3], m[7], m[11])
      out.m[15] =   det3x3( m[0], m[4], m[8], m[1], m[5], m[9], m[2], m[6], m[10])
    EndMethod
    Method Det4x4!()
      Local ans!
      ans =       m[0] * det3x3( m[5], m[9], m[13], m[6], m[10], m[14], m[7], m[11], m[15])
      ans = ans - m[1] * det3x3( m[4], m[8], m[12], m[6], m[10], m[14], m[7], m[11], m[15])
      ans = ans + m[2] * det3x3( m[4], m[8], m[12], m[5], m[11], m[13], m[7], m[11], m[15])
      ans = ans - m[3] * det3x3( m[4], m[8], m[12], m[5], m[11], m[13], m[6], m[10], m[14])
      return ans
    EndMethod

    Function Create:Matrix4(Ident = false)
      local mt:Matrix4 = new Matrix4
      if Ident = true
        mt.Identity()
      endif
      return mt
    EndFunction
  EndType

  'calculate the determinant of a 2x2 matrix.
  Function det3x3!( a1!, a2!, a3!, b1!, b2!, b3!, c1!, c2!, c3! )
    Local ans!
    ans = a1 * det2x2( b2, b3, c2, c3 )
    ans = ans - b1 * det2x2( a2, a3, c2, c3 )
    ans = ans + c1 * det2x2( a2, a3, b2, b3 )
    return ans
  EndFunction

  'calculate the determinant of a 2x2 matrix.
  Function det2x2!( a!, b!, c!, d!)
    Return a*d-b*c
  EndFunction

