Import BRL.LinkedList
import "Vector3.bmx"
private
	global _IPointx#
	global _IPointy#
	global _IPointz#
  global _IGrad#
  global _PlaneToll# = 0.01
Public

Function IntersectionX#()
  return _IPointX
EndFunction
Function IntersectionY#()
  return _IPointY
EndFunction
Function IntersectionZ#()
  return _IPointZ
EndFunction
Function IntersectionGrad#()
  return _IGrad
EndFunction

  Type Plane ' lists of planes are useful things
    field x#
    field y#
    field z#
    field d#
    Method this:Float Ptr()
      return varptr(x)
    EndMethod
    Method Zero()
      x = 0
      y = 0
      z = 0
      d = 0
    EndMethod
    Method Set(tx#,ty#,tz#,td#)
      x=tx
      y=ty
      z=tz
      d=td
    EndMethod
    
    Method Normalise()
      local d!=sqr(x*x+y*y+z*z)
      d=1.0/d
      x=x*d
      y=y*d
      z=z*d
    EndMethod
    Method Invert()
      x=-x
      y=-y
      z=-z
      d=-d
    EndMethod
    Method Crossv(ans:Vector3 Var , p:Plane)
      Local tx:Float, ty:Float ,tz:Float
      tx =y*p.z - z*p.y
      ty =z*p.x - x*p.z
      tz =x*p.y - y*p.x
      ans.x = tx
      ans.y = ty
      ans.z = tz
    EndMethod
    Method Dot#(p:Plane)
      Return (x*p.x + y*p.y + z*p.z)
    EndMethod
    Method Dotv#(p:Vector3)
      Return (x*p.x + y*p.y + z*p.z)
    EndMethod

    Method Compare(P:Plane,dead#=0.01)
      if abs(x-p.x)>dead return 0
      if abs(y-p.y)>dead return 0
      if abs(z-p.z)>dead return 0
      if abs(d-p.d)>dead return 0
      return 1
    EndMethod

    Method SetFromPoints(v1:Vector3, v2:Vector3, v3:Vector3)
      local c1:Vector3 = new Vector3
      local c2:Vector3 = new Vector3
      local normal:Vector3 = new Vector3
      v1.Sub(c1,v2)
      v3.Sub(c2,v2)
      c1.Cross(normal, c2)
      normal.Normalise()
      x=normal.x
      y=normal.y
      z=normal.z
      d=-normal.Dot(v2)
    EndMethod
    Method Copy( p:Plane )
        x = p.x
        y = p.y
        z = p.z
        d = p.d
    EndMethod
    Method Clone( p:Plane var )
      p.x = x
      p.y = y
      p.z = z
      p.d = d
    EndMethod
    Method DeriveAxis()
      local tx# = abs(x)
      local ty# = abs(y)
      local tz# = abs(z)
      if tx>ty and tx>tz return 1
      if ty>tx and ty>tz return 2
      return 3
    EndMethod
    Method Distance!(px!,py!,pz!)
    	Return ((x*px)+(y*py)+(z*pz)+d)/Sqr((x*x)+(y*y)+(z*z))
    EndMethod
    Method Distancev!(p:Vector3)
    	Return ((x*p.x)+(y*p.y)+(z*p.z)+d)/Sqr((x*x)+(y*y)+(z*z))
    EndMethod
    Method ClassifyPoint(px#,py#,pz#)
      local d# = Distance(px,py,pz)
      if d>-planetoll and d< planetoll 
        return 0 'OnPlane
      endif
      if d>-planetoll return  1                 'InFront
      if d< planetoll return -1                 'Behind
    EndMethod
    Method ClassifyPointv(p:Vector3)
      local d# = Distancev(p)
      if d>-planetoll and d< planetoll
        return 0 'OnPlane
      endif
      if d>-planetoll return  1                 'InFront
      if d< planetoll return -1                 'Behind
    EndMethod
    Method LineToPlane(p1x#,p1y#,p1z#,p2x#,p2y#,p2z#)
      local denom#=0
 	    'Calculate the position on the Line that intersects the plane
 	    denom= x * (p2x-p1x) + y * (p2y-p1y) + z * (p2z-p1z)
      If denom=0
 		    Return 3 'Line and plane don't intersect ie parallel
    	EndIf
      _IGrad = - (d + x * p1x + y * p1y + z * p1z) / denom
	    If _IGrad<0 Or _IGrad>1
    		Return 2
	    EndIf
	    _IPointx= p1x+ _IGrad * (p2x-p1x)
	    _IPointy= p1y+ _IGrad * (p2y-p1y)
	    _IPointz= p1z+ _IGrad * (p2z-p1z)
  	  Return 1
    EndMethod

    Function CreateFromPoints:Plane( v1:Vector3 , v2:Vector3 , v3:Vector3)
      local p:Plane = new Plane
      p.SetFromPoints( v1 , v2 , v3 )
      return p
    EndFunction
    Function Create:Plane(x#=0, y#=0, z#=0, d#=0)
      local p:Plane = new Plane
        p.x = x
        p.y = y
        p.z = z
        p.d = d
      return p
    EndFunction
  EndType

Function PlanesIntersect3(v:Vector3,n1:Plane,n2:Plane,n3:Plane)
  local n2n3x#,n2n3y#,n2n3z#
  local n3n1x#,n3n1y#,n3n1z#
  local n1n2x#,n1n2y#,n1n2z#
  local denom#

	n2.Crossv(v,n3)
	denom#=n1.Dotv(v)
	If denom = 0
		Return 0
	EndIf

 	n2.Crossv(v,n3)
	 n2n3x#=-n1.d * v.x / denom
	 n2n3y#=-n1.d * v.y / denom
	 n2n3z#=-n1.d * v.z / denom

 	n2.Crossv(v,n3)
	 n3n1x#=-n2.d * v.x / denom
	 n3n1y#=-n2.d * v.y / denom
	 n3n1z#=-n2.d * v.z / denom

 	n1.Crossv(v,n3)
	 n1n2x#=-n3.d * v.x / denom
	 n1n2y#=-n3.d * v.y / denom
	 n1n2z#=-n3.d * v.z / denom

	v.x = n2n3x+n3n1x+n1n2x
	v.y = n2n3y+n3n1y+n1n2y
	v.z = n2n3z+n3n1z+n1n2z
	_IPointx=v.x
	_IPointy=v.y
	_IPointz=v.z
	Return 1
End Function

global UpTemp:Plane = Plane.Create(0,1,0,0)
