
  Type Vector2
    Field x#,y#
    Method this:Float Ptr()
      return varptr(x)
    EndMethod

    Method Zero()
      x=0.0
      y=0.0
    EndMethod

    Method Set(tx#,ty#)
      x=tx
      y=ty
    EndMethod

    Method Dot#(v2:Vector2)
      return (x*v2.x + y*v2.y)
    EndMethod

    Method Magnitude:Float()
      return sqr( x*x + y*y )
    EndMethod

    Method MagnitudeSqr:Float()
      return ( x*x + y*y )
    EndMethod

    Method DeriveAxis()
      local tx# = abs(x)
      local ty# = abs(y)
      if tx>ty return Axis_IDX
      return Axis_IDY
    EndMethod
    Method Invert(v:Vector2 var)
      v.x=-x
      v.y=-y
    EndMethod
    Method GetCopy:Vector2()
      return Vector2.Create(self.x,self.y)
    EndMethod
    Method Copy(v:Vector2)
      x=v.x
      y=v.y
    EndMethod
    Method Clone( p:Vector2 var )
      p.x = x
      p.y = y
    EndMethod

    Method Swap(v:Vector2 var)
      local tmp#
      tmp=x
      x = v.x
      v.x=tmp
      tmp=y
      y = v.y
      v.y=tmp
    EndMethod

    Method Normalise()
      local l:Float
      l = sqr( x*x + y*y )
      If l = 0
        'Zero out the vector
        Zero()
      EndIf
      'Invert and Multiply
      l=1.0 / l
      x = x * l
      y = y * l
    EndMethod

    Method Compare(v1:Vector2,toll#=0.0003)
      if abs(v1.x-x)>toll return False
      if abs(v1.y-y)>toll return False
      Return True
    EndMethod
    Method Add(ans:Vector2 Var , v2:Vector2)
      ans.x = x + v2.x
      ans.y = y + v2.y
    EndMethod
    Method Sub(ans:Vector2 Var , v2:Vector2)
      ans.x = x - v2.x
      ans.y = y - v2.y
    EndMethod
    Method AddScalar(ans:Vector2 Var , val:Float)
      ans.x = x + val
      ans.y = y + val
    EndMethod
    Method SubScalar(ans:Vector2 Var , val:Float)
      ans.x = x - val
      ans.y = y - val
    EndMethod
    Method MulScalar(ans:Vector2 Var , val:Float)
      ans.x = x * val
      ans.y = y * val
    EndMethod
    Method DivScalar(ans:Vector2 Var , val:Float)
      ans.x = x / val
      ans.y = y / val
    EndMethod
    Method Rotate(ans:Vector2 Var ,theta#)
      Local qx#, qy#
      local cs# = Cos(theta)
      local sn# = Sin(theta)
    	qx#=  x * Cs + y * Sn
    	qy#= -x * Sn + y * Cs
     	ans.x = qx
     	ans.y = qy
    EndMethod

    Method RotateOrigin(ans:Vector2 Var, Origin:Vector2, theta#)
      Local qx#, qy#
      local cs# = Cos(theta)
      local sn# = Sin(theta)
      x = x - Origin.x
      y = y - Origin.y
    	qx#=  x * Cs + y * Sn
    	qy#= -x * Sn + y * Cs
     	ans.x = qx - Origin.x
     	ans.y = qy + Origin.y
    EndMethod

    Function Create:Vector2(tx#=0, ty#=0)
      local nv:Vector2 = new Vector2
        nv.x=tx
        nv.y=ty
        return nv
    EndFunction
  EndType

