import "Interpolate.bmx"

  Type Vector3
    'Data fields
    Field x#,y#,z#

    Method this:Float Ptr()
      return varptr(x)
    EndMethod

    Method Min( ans:Vector3 var, wi:vector3 )
      if x>wi.x ans.x = wi.x else ans.x=x
      if y>wi.y ans.y = wi.y else ans.y=y
      if z>wi.z ans.z = wi.z else ans.z=z
    EndMethod

    Method Max( ans:Vector3 var, wi:vector3 )
      if x<wi.x ans.x=wi.x else ans.x=x
      if y<wi.y ans.y=wi.y else ans.y=y
      if z<wi.z ans.z=wi.z else ans.z=z
    EndMethod

    Method Distance#(to_:Vector3)
      return sqr(self.DistanceSqr(to_))
    EndMethod

    Method DistanceSqr#(to_:Vector3)
      local dx#=x-to_.x
      local dy#=y-to_.y
      local dz#=z-to_.z
      return (dx*dx+dy*dy+dz*dz)
    EndMethod

    Method Zero()
      x=0.0
      y=0.0
      z=0.0
    EndMethod

    Method Set(tx#,ty#,tz#)
      x=tx
      y=ty
      z=tz
    EndMethod
    Method Copy(v:Vector3)
      x=v.x
      y=v.y
      z=v.z
    EndMethod
    Method GetCopy:Vector3()
      return Vector3.Create(self.x,self.y,self.z)
    EndMethod
    Method Clone( v:Vector3 var )
      v.x = x
      v.y = y
      v.z = z
    EndMethod

    Method Magnitude:Float()
      return sqr( self.MagnitudeSqr() )
    EndMethod

    Method MagnitudeSqr:Float()
      return ( x*x + y*y + z*z )
    EndMethod

    Method DeriveAxis()
      local tx# = abs(x)
      local ty# = abs(y)
      local tz# = abs(z)

      if tx>ty and tx>tz return Axis_IDX
      if ty>tx and ty>tz return Axis_IDY
      return Axis_IDZ
    EndMethod

    Method Invert(v:Vector3 var)
      v.x=-x
      v.y=-y
      v.z=-z
    EndMethod
    Method Swap(v:Vector3 var)
      local tmp#
      tmp=x
      x = v.x
      v.x=tmp
      tmp=y
      y = v.y
      v.y=tmp
      tmp=z
      z = z
      v.z=tmp
    EndMethod

    Method Normalise()
      local l:Float
      l = sqr( x*x + y*y + z*z )
      If l = 0 return
      'Invert and Multiply
      l=1.0 / l
      x = x * l
      y = y * l
      z = z * l
    EndMethod

    Method Cross(ans:Vector3 Var , v2:Vector3)
      Local tx:Float, ty:Float ,tz:Float
      tx =y*v2.z - z*v2.y
      ty =z*v2.x - x*v2.z
      tz =x*v2.y - y*v2.x
      ans.x = tx
      ans.y = ty
      ans.z = tz
    EndMethod

    Method Add(ans:Vector3 Var , v2:Vector3)
      ans.x = x + v2.x
      ans.y = y + v2.y
      ans.z = z + v2.z
    EndMethod

    Method Sub(ans:Vector3 Var , v2:Vector3)
      ans.x = x - v2.x
      ans.y = y - v2.y
      ans.z = z - v2.z
    EndMethod
    Method AddScalar(ans:Vector3 Var , val:Float)
      ans.x = x + val
      ans.y = y + val
      ans.z = z + val
    EndMethod
    Method SubScalar(ans:Vector3 Var , val:Float)
      ans.x = x - val
      ans.y = y - val
      ans.z = z - val
    EndMethod
    Method MulScalar(ans:Vector3 Var , val:Float)
      ans.x = x * val
      ans.y = y * val
      ans.z = z * val
    EndMethod
    Method DivScalar(ans:Vector3 Var , val:Float)
      ans.x = x / val
      ans.y = y / val
      ans.z = z / val
    EndMethod

    Method Dot#(v2:Vector3)
      Return (x*v2.x + y*v2.y + z*v2.z)
    EndMethod

    Method MidPoint(ans:Vector3 Var , v2:Vector3)
      ans.x = x+((v2.x-x)/2.0)
      ans.y = y+((v2.y-y)/2.0)
      ans.z = z+((v2.z-z)/2.0)
    EndMethod

    Method RotateX(ans:Vector3 Var ,theta#)
      Local qx#, qy#, qz#
      local cs# = Cos(theta)
      local sn# = Sin(theta)
    	qx#=  x
     	qy#=  y * Cs + z * Sn
     	qz#= -y * Sn + z * Cs
     	ans.x = qx
     	ans.y = qy
     	ans.z = qz
    EndMethod

    Method RotateY(ans:Vector3 Var ,theta#)
      Local qx#, qy#, qz#
      local cs# = Cos(theta)
      local sn# = Sin(theta)
    	qx#=  x * Cs + z * Sn
    	qy#=  y
    	qz#= -x * Sn + z * Cs
     	ans.x = qx
     	ans.y = qy
     	ans.z = qz
    EndMethod

    Method RotateZ(ans:Vector3 Var ,theta#)
      Local qx#, qy#, qz#
      local cs# = Cos(theta)
      local sn# = Sin(theta)
    	qx#=  x * Cs + y * Sn
    	qy#= -x * Sn + y * Cs
    	qz#=  z
     	ans.x = qx
     	ans.y = qy
     	ans.z = qz
    EndMethod

    'Rotate a point p by angle theta around an arbitrary normal r
    'Return the rotated point.
    'Positive angles are anticlockwise looking down the axis
    'towards the Origin.
    'Assume Right hand coordinate system.
    Method RotateAxis(ans:Vector3 Var, r:Vector3, theta#)
    	Local costheta#,sintheta#
    	Local q:Vector3 = New Vector3
    	q.Zero()
      r.Normalise()
      if r.MagnitudeSqr() = 0 return

    	costheta = Cos(theta)
    	sintheta = Sin(theta)

    	q.x=(costheta + (1.0 - costheta) * r.x * r.x) * x
    	q.x:+( ((1.0 - costheta) * r.x * r.y - r.z * sintheta) * y)
    	q.x=q.x +( ((1.0 - costheta) * r.x * r.z + r.y * sintheta) * z)

    	q.y=q.y +( ((1.0 - costheta) * r.x * r.y + r.z * sintheta) * x)
    	q.y=q.y +( (costheta + (1 - costheta) * r.y * r.y) * y)
    	q.y=q.y +( ((1.0 - costheta) * r.y * r.z - r.x * sintheta) * z)

    	q.z=q.z +( ((1.0 - costheta) * r.x * r.z - r.y * sintheta) * x)
    	q.z=q.z +( ((1.0 - costheta) * r.y * r.z + r.x * sintheta) * y)
    	q.z=q.z +( (costheta + (1.0 - costheta) * r.z * r.z) * z)
    	ans.Copy(q)
    EndMethod

    Method CosineInterp(ans:Vector3 var, v1:Vector3, d#)
      local q:Vector3 = new Vector3
      q.x = CosineInterpolate( x, v1.x, d )
      q.y = CosineInterpolate( y, v1.y, d )
      q.z = CosineInterpolate( z, v1.z, d )
      ans.Copy(q)
    EndMethod
    Method LinearInterp(ans:Vector3 var, v1:Vector3, d#)
      local q:Vector3 = new Vector3
      q.x = LinearInterpolate( x, v1.x, d )
      q.y = LinearInterpolate( y, v1.y, d )
      q.z = LinearInterpolate( z, v1.z, d )
      ans.Copy(q)
    EndMethod

    Method Compare(v1:Vector3,toll#=0.0003)
      if abs(v1.x-x)>toll return False
      if abs(v1.y-y)>toll return False
      if abs(v1.z-z)>toll return False
      Return True
    EndMethod

    Function Create:Vector3(tx#=0, ty#=0, tz#=0)
      local nv:Vector3 = new Vector3
        nv.x=tx
        nv.y=ty
        nv.z=tz
        return nv
    EndFunction
  EndType

Function Closest:Vector3(a:Vector3, b:Vector3, c:Vector3, p:Vector3)
  local da#=a.Distancesqr(p)
  local db#=b.Distancesqr(p)
  local dc#=c.Distancesqr(p)
  if da<db and da<dc return a
  if db<da and db<dc return b
  if dc<da and dc<db return c
  return a
EndFunction

Function ClosestPointOnTriangle:Vector3(a:Vector3, b:Vector3, c:Vector3, p:Vector3)
  local Rab:Vector3 = closestPointOnLine(a, b, p)
  Local Rbc:Vector3 = closestPointOnLine(b, c, p)
  Local Rca:Vector3 = closestPointOnLine(c, a, p)
  return closest(Rab, Rbc, Rca, p)
EndFunction



Function closestPointOnLine:Vector3( a:Vector3, b:Vector3, p:Vector3)
  'Determine t (the length of the vector from a to p)
  local c:vector3=new Vector3
  Local V:Vector3=new Vector3
  Local d!
  local t!

  p.Sub(c,a)
  b.Sub(V,a)
  V.Normalise()

  d=a.Distance(b)
  t=V.Dot(c)

  'Check to see if t is beyond the extents of the line segment
   if t<0 return a.GetCopy()
   if t>d return b.GetCopy()

   'Return the point between a and b
   'set length of V to t
   V.MulScalar(V,t)
   a.Add(c,V)
   return c
EndFunction

Function intersectSphere#(rO:Vector3, rV:Vector3, sO:Vector3, sR#)
  local Q:Vector3 = New Vector3
  local c#,v#,d#
  sO.Sub(Q,r0)
  c=Q.Magnitude()
  v=Q.dot(rV)
  d=sR*sR -(c*c-v*v)
  'If there was no intersection, return -1
  if d<0.0 return -1.0
  'Return the distance to the first intersecting point
  return v - sqr(d)
EndFunction




