' bpaint.bmx
' simple blitzmax paint program
' by simonarmstrong@blitzbasic.com

' pan the zoomed area by dragging the mouse in the left window
' draw to the image using the left and mouse buttons in the right window
' use the console to load and save images, click in the console and type
' one of the following commands: Open, New, Save, Exit, Quit, CD, Dir, Help

' note the use of function pointers to implement the text console and the
' WriteBMP function to save an image to a file

Strict

InitBTree

While Not KeyHit(KEY_ESCAPE)
	UpdateBTree
	RenderBTree
	Flip
	FlushMem
Wend

End

' btree project manager

Global font,screenwidth,screenheight,menuheight,menuview,treewidth,consoleheight
Global pointerx,pointery,mouseevent,mouseover,keyboardfocus

Function InitBTree()
	screenwidth=800
	screenheight=600
	consoleheight=110
	treewidth=256
	Graphics screenwidth,screenheight
'	font=LoadImageFont2("bpaint.fnt")

'	font=LoadImageFont("Blitz.fon",9)
'	If font SetImageFont font

	menuheight=TextHeight(" ")
	If menuheight=0 menuheight=40
	SetBlend ALPHABLEND
	InitConsole
	InitBPaint
End Function

Function UpdateBTree()
	pointerx=MouseX()
	pointery=MouseY()
	If pointery<menuheight
		UpdateMenuBar pointerx,pointery
		Return
	EndIf
	If pointery>screenheight-consoleheight
		UpdateConsole pointerx,pointery-(screenheight-consoleheight)
		Return
	EndIf
	UpdateBPaint pointerx,pointery-menuheight
End Function

Function RenderBTree()
	SetOrigin 0,0		
	SetViewport 0,0,screenwidth,screenheight
	SetScale 1,1
	Cls
	RenderMenuBar	
	RenderConsole 0,screenheight-consoleheight,screenwidth,consoleheight
	RenderBPaint 0,menuheight,screenwidth,screenheight-menuheight-consoleheight
End Function

Function RenderMenuBar()
	SetColor 0,0,255
	DrawRect 0,0,screenwidth,menuheight
	SetColor 255,255,255
	If pointery<menuheight
		DrawText "File  View  Image  Help",4,0
	Else
		DrawText "BPaint Image Editor "+MemAlloced(),4,0
	EndIf		
	DrawText "X",screenwidth-14,0
End Function

Function UpdateMenuBar(mx,my)
End Function

' console with command line interpreter

Global CommandNames$[],CommandFunctions%(arg$)[],CommandCount

Function AddCommand(name$,func%(arg$))
	Local	n
	n=Len(CommandNames)
	If CommandCount>=n
		n=CommandCount+16
		CommandNames=CommandNames[..n+16]
		CommandFunctions=CommandFunctions[..n+16]
	EndIf
	CommandNames[CommandCount]=Lower$(name)
	CommandFunctions[CommandCount]=func
	CommandCount:+1
End Function

Function CallCommand(name$,arg$)
	Local	i
	name=Lower$(name)
	For i=0 Until CommandCount
		If CommandNames$[i]=name
			Return CommandFunctions[i](arg)
		EndIf
	Next
	Return -1
End Function

Function ParseArg$(a$ Var)
	Local 	p,r$
	p=a.Find(",")+1
	If p=0 p=Len a$+1
	r$=a$[..p-1]
	a$=a$[p..]
	Return r$	
End Function

Global conout$,conin$,conlines,conscroll,conwidth,conheight,cursorx

Function ExitApplication(a$)
	End
End Function

Function ListCurrentDir(a$)
	Local	path$,dir,f$
	path=CurrentDir()
	WriteConsole "CurrentDir="+path
	dir=ReadDir(path)
	If dir
		While True
			f$=NextFile(dir)
			If Not f$ Exit
			WriteConsole f$
		Wend
		CloseDir dir
	EndIf	
End Function

Function ChangeCurrentDir(a$)
	ChangeDir a$
	WriteConsole CurrentDir()
End Function

Function ListCommands(a$)
	Local	i
	For i=0 Until CommandCount
		WriteConsole CommandNames$[i]
	Next
End Function

Function OpenCommand(a$)
	Return OpenBPaint(a$)	
End Function

Function SaveCommand(a$)
	Return SaveBPaint(a$)	
End Function

Function SnapshotScreen(a$)
	Local image
	image=CreateImage(GraphicsWidth(),GraphicsHeight())
	GrabImage image,0,0
	If a$ SaveImage image,a$
End Function

Function NewCommand(a$)
	Local	w,h
	w=Int(ParseArg(a$))
	h=Int(ParseArg(a$))
	If w And h
		Return NewBPaint(w,h)	
	Else
		WriteConsole "<usage> new width,height"
	EndIf
End Function

Function InitConsole()
	AddCommand "Open",OpenCommand
	AddCommand "New",NewCommand
	AddCommand "Save",SaveCommand
	AddCommand "Exit",ExitApplication
	AddCommand "Quit",ExitApplication
	AddCommand "CD",ChangeCurrentDir
	AddCommand "Dir",ListCurrentDir
	AddCommand "Help",ListCommands
	AddCommand "Snapshot",SnapshotScreen
	conscroll=32768
	WriteConsole "BTree v0.1 Console"
	conin=">"
	cursorx=1
End Function

Function WriteConsole(a$)
	Local	p
	conout$:+Trim(a$)+Chr(10)
	conlines=0
	p=1
	While True
		p=Instr(conout,Chr(10),p)
		If p<=0 Exit
		p=p+1
		conlines:+1
	Wend
End Function

Function RenderConsole(viewx,viewy,vieww,viewh)
	Local	lines,y,t,p,q,x,s$,lineheight
	lineheight=TextHeight(" ")
	If lineheight=0 lineheight=8
	SetOrigin viewx,viewy
	SetViewport viewx,viewy,vieww,viewh
' background
	SetColor 0,0,255
	DrawRect 0,0,vieww,viewh
' text
	SetColor 255,255,255
	SetOrigin viewx+4,viewy
	lines=viewh/lineheight-1
	t=conscroll
	If t>conlines-lines t=conlines-lines
	If t<0 t=0
' find first line
	p=1
	While t
		p=Instr(conout,Chr(10),p)
		If p<=0 p=1;Exit
		p:+1
		t:-1
	Wend
' print
	y=0	
	While lines>0
		q=Instr(conout,Chr(10),p)
		If q<=0 Exit
		DrawText conout[p-1..q],0,y
		lines:-1
		y:+lineheight
		p=q+1
	Wend
' input line
	s$=conin[..cursorx]
	x=TextWidth(s$)
	DrawText s$,0,y
	DrawRect x+1,y+3,2,lineheight-3
	DrawText conin[cursorx..],x+2,y
End Function

Function UpdateConsole(mx,my)
	Local	c
	c=getchar()
	If c 
		conin$=conin[..cursorx]+Chr(c)+conin[cursorx..]
		cursorx:+1
		Return
	EndIf
	
	If KeyHit(KEY_BACKSPACE)
		If cursorx>1 
			conin$=conin[..cursorx-1]+conin[cursorx..]
			cursorx:-1
		EndIf
	EndIf

	If KeyHit(KEY_DELETE)
		If cursorx<Len(conin$) 
			conin$=conin[..cursorx]+conin[cursorx+1..]
		EndIf
	EndIf
	
	If KeyHit(KEY_LEFT)
		If cursorx>1 
			cursorx:-1
		EndIf
	EndIf
	
	If KeyHit(KEY_RIGHT)
		If cursorx<Len(conin$) 
			cursorx:+1
		EndIf
	EndIf
	
	If KeyHit(KEY_ENTER)
		WriteConsole conin
		ConsoleCommand conin
		conin=">"
		cursorx=1
	EndIf

End Function

Function ConsoleCommand(cmd$)
	Local	t,arg$
	cmd=cmd$[1..]
	t=Instr(cmd," ")
	If t>0
		arg=cmd[t..]
		cmd=cmd[..t-1]
	EndIf
	Return CallCommand(cmd,arg)	
End Function


' bpaint image editor

Global image:TImage,imagedirty,checkers
Global brushx,brushy,oldbrushx,oldbrushy
Global zoom,zoomx,zoomy,zoomw,zoomh,maxzoomx,maxzoomy,panx,pany,panspeed
Global bpaintwidth,bpaintheight,toolwidth,split
Global oldmx,oldmy,oldml,oldmr

' mousestates

Const MOUSEHOVER=0
Const MOUSELCLICK=1
Const MOUSERCLICK=2
Const MOUSELDRAG=3
Const MOUSERDRAG=4
Const MOUSELRELEASE=5
Const MOUSERRELEASE=6

' mouseovers

Const OVERNOTHING=0
Const OVERIMAGE=1
Const OVERSPLIT=2
Const OVERZOOM=3
Const OVERTOOLBAR=4

Function InitBPaint()
	toolwidth=160
	zoom=4
	panspeed=4
	OpenBPaint("penguin.png")
	checkers=LoadImage("checkers.png")
	split=ImageWidth(image)
End Function

Function NewBPaint(w,h)
	image=CreateImage(w,h,DYNAMICIMAGE)
	GrabImage image,0,0
	Local pix=LockImage(image)
	WritePixelRect pix,0,0,w,h,0
	UnlockImage(image)
End Function

Function OpenBPaint(filename$)
	If filename image=LoadImage(filename,DYNAMICIMAGE)
End Function

Function SaveBPaint(filename$)
	If filename SaveImage(image,filename)
End Function

Function RenderBPaint(viewx,viewy,vieww,viewh)	
	RethinkBPaint vieww,viewh
' splitter
	SetOrigin viewx,viewy
	SetViewport viewx,viewy,vieww,viewh		
	SetColor 255,255,255
	DrawRect split,0,4,viewh
' normal view	
	SetOrigin viewx,viewy
	SetViewport viewx,viewy,split,viewh
	SetColor 255,255,255
	DrawImage image,-panx,-pany
	SetAlpha 0.5
	SetColor 0,0,0
	drawbox zoomx-1,zoomy-1,zoomw+1,zoomh+1
	SetColor 255,255,255
	drawbox zoomx-2,zoomy-2,zoomw+3,zoomh+3
	SetAlpha 1
' zoom view	
	SetOrigin viewx+split+4,viewy
	SetViewport viewx+split+4,viewy,vieww-toolwidth-split+4,viewh
'	TileImage checkers,0,0
	SetScale zoom,zoom
	DrawImage image,-zoomx*zoom,-zoomy*zoom
	SetScale 1,1
' toolbar	
	SetOrigin viewx+vieww-toolwidth,viewy
	SetViewport viewx+vieww-toolwidth,viewy,toolwidth,viewh
	SetColor 128,128,128
	DrawRect 0,0,toolwidth,viewh
End Function

Function RethinkBPaint(swidth,sheight)
	Local	w,h,zw,zh,pw,ph
	bpaintwidth=swidth
	bpaintheight=sheight
	If split>swidth/2 split=swidth/2
	w=ImageWidth(image)
	h=ImageHeight(image)
	zoomw=(swidth-toolwidth-(split+4))/zoom
	zoomh=(sheight-menuheight)/zoom
	zw=w-zoomw
	zh=h-zoomh
	If zw<0 zw=0
	If zh<0 zh=0	
	If zoomx<0 zoomx=0
	If zoomy<0 zoomy=0
	If zoomx>zw zoomx=zw
	If zoomy>zh zoomy=zh
	panx=zoomx
	pany=zoomy
	pw=w-split
	ph=h-(sheight-menuheight)
	If pw<0 pw=0
	If ph<0 ph=0
	If panx>pw panx=pw
	If pany>ph pany=ph	
	If zoomw>w zoomw=w
	If zoomh>h zoomh=h	
End Function

Function UpdateBPaint(mx,my)
	Local ml,mr
	Local pix:TPixmap
' handle keyboard events
	If KeyHit(KEY_EQUALS) And zoom<32 zoom:+1
	If KeyHit(KEY_MINUS) And zoom>1 zoom:-1
	If KeyHit(KEY_LEFT) zoomx:-panspeed
	If KeyHit(KEY_UP) zoomy:-panspeed
	If KeyHit(KEY_RIGHT) zoomx:+panspeed
	If KeyHit(KEY_DOWN) zoomy:+panspeed
' detect mouse events
'	mx=MouseX()
'	my=MouseY()
	ml=KeyDown(1)
	mr=KeyDown(2)
	mouseevent=MOUSEHOVER	
	If mx<>oldmx Or my<>oldmy 
		If ml mouseevent=MOUSELDRAG
		If mr mouseevent=MOUSERDRAG
	EndIf
	If ml And (Not oldml) mouseevent=MOUSELCLICK
	If mr And (Not oldmr) mouseevent=MOUSERCLICK
	If (Not ml) And oldml mouseevent=MOUSELRELEASE
	If (Not mr) And oldmr mouseevent=MOUSERRELEASE
	oldmx=mx
	oldmy=my
	oldml=ml
	oldmr=mr
' update mouseover
	If mx<split
		mouseover=OVERIMAGE
		brushx=mx+panx
		brushy=my+pany
	Else
		If mx>=bpaintwidth-toolwidth
			mouseover=OVERTOOLBAR
		Else
			If mx>split+4
				mouseover=OVERZOOM
				mx:-split+4
				brushx=mx/zoom+zoomx
				brushy=my/zoom+zoomy
			EndIf
		EndIf
	EndIf
' handle mouse
	Select mouseover
		Case OVERIMAGE
			If mouseevent=MOUSELCLICK Or mouseevent=MOUSELDRAG
				zoomx=brushx-zoomw/2
				zoomy=brushy-zoomh/2
			EndIf
		Case OVERZOOM
			Select mouseevent
				Case MOUSELCLICK
					pix=LockImage(image)
					WritePixel pix,brushx,brushy,-1
					UnlockImage(image)
				Case MOUSERCLICK
					pix=LockImage(image)
					WritePixel pix,brushx,brushy,0
					UnlockImage(image)
				Case MOUSELDRAG
					pix=LockImage(image)
					WritePixelLine pix,oldbrushx,oldbrushy,brushx,brushy,-1
					UnlockImage(image)
				Case MOUSERDRAG
					pix=LockImage(image)
					WritePixelLine pix,oldbrushx,oldbrushy,brushx,brushy,0
					UnlockImage(image)
			End Select
	End Select
	oldbrushx=brushx
	oldbrushy=brushy
End Function
		

Function WritePixelLine(pix:TPixmap,x0,y0,x1,y1,argb)
	Local	a,b	
	a=Abs(x1-x0)
	b=Abs(y1-y0)
	If a<b a=b
	If a=0
		WritePixel pix,x0,y0,argb
	Else
		For b=0 To a
			WritePixel pix,x0+(b*(x1-x0))/a,y0+(b*(y1-y0))/a,argb
		Next
	EndIf
End Function

Function WritePixelRect(pix:TPixmap,x,y,w,h,argb)
	Local	u,v
	For v=0 Until h
		For u=0 Until w
			WritePixel pix,x+u,y+v,argb
		Next
	Next
End Function

Function DrawBox(x#,y#,w#,h#)
	DrawRect x,y,w,1
	DrawRect x,y+h-1,w,1
	DrawRect x,y+1,1,h-2
	DrawRect x+w-1,y+1,1,h-2
End Function

Function getchar()
	Local i,shft
	If KeyHit(32) Return 32
	If KeyDown(KEY_SHIFT)
		For i=0 To 9
			If KeyHit(48+i)
				Return Asc(Mid$(")!@#$%^&*()",i+1,1))
			EndIf
		Next
		For i=0 To 25
			If KeyHit(65+i) 
				Return 65+i
			EndIf
		Next
		If KeyHit(KEY_TILDE) Return Asc("~")
		If KeyHit(KEY_MINUS) Return Asc("_")
		If KeyHit(KEY_EQUALS) Return Asc("+")
		If KeyHit(KEY_OPENBRACKET) Return Asc("{")
		If KeyHit(KEY_CLOSEBRACKET) Return Asc("}")
		If KeyHit(KEY_BACKSLASH) Return Asc("|")
		If KeyHit(KEY_SEMICOLON) Return Asc(":")
		If KeyHit(KEY_QUOTES) Return 34
		If KeyHit(KEY_COMMA) Return Asc("<")
		If KeyHit(KEY_PERIOD) Return Asc(">")
		If KeyHit(KEY_SLASH) Return Asc("?")	
	Else
		For i=0 To 9
			If KeyHit(48+i)
				Return 48+i
			EndIf
		Next
		For i=0 To 25
			If KeyHit(65+i) 
				Return 97+i
			EndIf
		Next
		If KeyHit(KEY_TILDE) Return Asc("`")
		If KeyHit(KEY_MINUS) Return Asc("-")
		If KeyHit(KEY_EQUALS) Return Asc("=")
		If KeyHit(KEY_OPENBRACKET) Return Asc("[")
		If KeyHit(KEY_CLOSEBRACKET) Return Asc("]")
		If KeyHit(KEY_BACKSLASH) Return Asc("\")
		If KeyHit(KEY_SEMICOLON) Return Asc(";")
		If KeyHit(KEY_QUOTES) Return Asc("'")
		If KeyHit(KEY_COMMA) Return Asc(",")
		If KeyHit(KEY_PERIOD) Return Asc(".")
		If KeyHit(KEY_SLASH) Return Asc("/")	
	EndIf
End Function

Function SaveImage(image:TImage,filename$)
	Local stream:TStream
	stream=WriteStream(filename)
	If stream
		WriteBMP stream,image.masks[0]
		CloseStream stream
	EndIf
End Function

Function WriteBMP( stream:TStream,pixmap:TPixmap )
	Local	buf:Byte[1024]
	Local	hsize,hoffset
	Local	size,width,height
	Local	planes,bits,compression,isize,xpels,ypels,cols,inuse
	Local	w,y
	
	width=pixmap.width
	height=pixmap.height
	w=width*3
	w=(w+3)&$fffc
	hsize=w*height+54
	size=40
	hoffset=54
	planes=1
	bits=24
	compression=0
	isize=40
	xpels=2834
	ypels=2834
	cols=0
	inuse=0
	stream=LittleEndianStream( stream )
	WriteByte stream,Asc("B")
	WriteByte stream,Asc("M")
	WriteInt stream,hsize
	WriteInt stream,0
	WriteInt stream,hoffset
	WriteInt stream,size
	WriteInt stream,width
	WriteInt stream,height
	WriteShort stream,planes
	WriteShort stream,bits
	WriteInt stream,compression
	WriteInt stream,isize
	WriteInt stream,xpels
	WriteInt stream,ypels
	WriteInt stream,cols
	WriteInt stream,inuse	
	buf=New Byte[w]
	For y=height-1 To 0 Step -1
		ConvertPixels(pixmap.pixelptr(0,y),pixmap.format,buf,PF_BGR888,width)
		stream.WriteBytes(buf,w)		
	Next
End Function

Function CopyImage:TImage( src:TImage,x,y,width,height )
	Local image:TImage
	Local srcpix:TPixmap,destpix:TPixmap
	Local format,i		

	srcpix=src.masks[0]
	format=srcpix.format
	destpix=CreatePixmap(width,height,format)	
	For i=0 Until height
		CopyPixels srcpix.PixelPtr(x,y+i),destpix.PixelPtr(0,i),format,width
	Next		
	image=TImage.Alloc( width,height,1,FILTEREDIMAGE )		
	image.frames[0]=BRL.Max2D.Blitz2D_Driver.CreateFrameFromPixmap( destpix,FILTEREDIMAGE )
	image.masks[0]=destpix
	Return image
End Function

Rem

Function LoadImageFont2:TImageFont(url$)
	Local	img:TImage
	Local	fnt:TImageFont
	Local	glyph:TImageGlyph
	Local	pages,x,y,c
	
	img=LoadImage(url$)
	If Not img Return

	fnt=New TImageFont
	fnt.name=url
	fnt.src=url
	fnt.height=8
	fnt.ascend=6
	fnt.descend=2
	c=32
	For y=0 To 2
		For x=0 To 31
			glyph=New TImageGlyph
			glyph.char=c
			glyph.offx=0
			glyph.offy=-5
			glyph.advance=4 Shl 6
			glyph.page=pages
			fnt.glyph[glyph.char]=glyph
			If c>32 
				glyph.width=4
				glyph.height=8
				fnt.page[pages]=CopyImage(img,x*4,y*8,4,8)
				pages:+1
			EndIf
			c:+1
		Next
	Next
	fnt.pages=pages	
	If c>89 And c<97
		For c=1 To 26
			fnt.glyph[96+c]=fnt.glyph[64+c]
		Next
	EndIf
			
	Return fnt
End Function

endrem